""""""

import aiohttp
import aiounifi
import argparse
import asyncio
import async_timeout
import logging

LOGGER = logging.getLogger(__name__)


def signalling_callback(signal, data):
    LOGGER.info(f"{signal}, {data}")


async def unifi_controller(
    host, username, password, port, site, session, sslcontext, callback
):
    """Setup UniFi controller and verify credentials."""
    controller = aiounifi.Controller(
        host,
        username=username,
        password=password,
        port=port,
        site=site,
        websession=session,
        sslcontext=sslcontext,
        callback=callback,
    )

    try:
        with async_timeout.timeout(10):
            await controller.login()
        return controller

    except aiounifi.Unauthorized:
        LOGGER.warning(f"Connected to UniFi at {host} but not registered")

    except (asyncio.TimeoutError, aiounifi.RequestError):
        LOGGER.error(f"Error connecting to the UniFi controller at {host}")

    except aiounifi.AiounifiException:
        LOGGER.exception("Unknown UniFi communication error occurred")


async def main(host, username, password, port, site, sslcontext=False):
    """Main function."""
    LOGGER.info("Starting aioUniFi")
    loop = asyncio.get_event_loop()

    websession = aiohttp.ClientSession(
        loop=loop, cookie_jar=aiohttp.CookieJar(unsafe=True)
    )

    controller = await unifi_controller(
        host=host,
        username=username,
        password=password,
        port=port,
        site=site,
        session=websession,
        sslcontext=sslcontext,
        callback=signalling_callback,
    )

    if not controller:
        LOGGER.error("Couldn't connect to UniFi controller.")
        return

    await controller.initialize()
    await controller.sites()
    controller.start_websocket()

    try:
        while True:
            await asyncio.sleep(1)

    except KeyboardInterrupt:
        pass

    finally:
        controller.stop_websocket()
        await controller.session.close()


if __name__ == "__main__":
    logging.basicConfig(format="%(message)s", level=logging.DEBUG)
    parser = argparse.ArgumentParser()
    parser.add_argument("host", type=str)
    parser.add_argument("username", type=str)
    parser.add_argument("password", type=str)
    parser.add_argument("-p", "--port", type=int, default=8443)
    parser.add_argument("-s", "--site", type=str, default="default")
    args = parser.parse_args()
    LOGGER.info(
        "%s %s %s %s %s", args.host, args.username, args.password, args.port, args.site
    )
    asyncio.run(
        main(
            host=args.host,
            username=args.username,
            password=args.password,
            port=args.port,
            site=args.site,
        )
    )



"""Event messages on state changes."""

#  https://demo.ui.com/manage/locales/en/eventStrings.json?v=5.4.11.2

CONTROLLER_UPDATE_AVAILABLE = "EVT_AD_UPDATE_AVAILABLE"

ACCESS_POINT_ADOPTED = "EVT_AP_ADOPTED"
ACCESS_POINT_CONFIGURED = "EVT_AP_CONFIGURED"
ACCESS_POINT_CONNECTED = "EVT_AP_CONNECTED"
ACCESS_POINT_DELETED = "EVT_AP_DELETED"
ACCESS_POINT_RESTARTED = "EVT_AP_RESTARTED"
ACCESS_POINT_UPGRADED = "EVT_AP_UPGRADED"

GATEWAY_ADOPTED = "EVT_GW_ADOPTED"
GATEWAY_CONNECTED = "EVT_GW_CONNECTED"
GATEWAY_DELETED = "EVT_GW_DELETED"
GATEWAY_LOST_CONTACT = "EVT_GW_LOST_CONTACT"
GATEWAY_RESTART = "EVT_GW_RESTARTED"
GATEWAY_UPGRADED = "EVT_GW_UPGRADED"

SWITCH_ADOPTED = "EVT_SW_ADOPTED"
SWITCH_CONNECTED = "EVT_SW_CONNECTED"
SWITCH_DELETED = "EVT_SW_DELETED"
SWITCH_LOST_CONTACT = "EVT_SW_LOST_CONTACT"
SWITCH_OVERHEAT = "EVT_SW_OVERHEAT"
SWITCH_POE_OVERLOAD = "EVT_SW_POE_OVERLOAD"
SWITCH_POE_DISCONNECT = "EVT_SW_POE_DISCONNECT"
SWITCH_RESTARTED = "EVT_SW_RESTARTED"
SWITCH_UPGRADED = "EVT_SW_UPGRADED"

CLIENT_BLOCKED = "EVT_WC_BLOCKED"
CLIENT_UNBLOCKED = "EVT_WC_UNBLOCKED"
WIRED_CLIENT_CONNECTED = "EVT_LU_CONNECTED"
WIRED_CLIENT_DISCONNECTED = "EVT_LU_DISCONNECTED"
WIRELESS_CLIENT_CONNECTED = "EVT_WU_CONNECTED"
WIRELESS_CLIENT_DISCONNECTED = "EVT_WU_DISCONNECTED"
WIRELESS_GUEST_CONNECTED = "EVT_WG_CONNECTED"
WIRELESS_GUEST_DISCONNECTED = "EVT_WG_DISCONNECTED"


class event:
    def __init__(self, raw):
        self.raw = raw

    @property
    def event(self):
        return self.raw["key"]

    @property
    def msg(self):
        return self.raw["msg"]

    @property
    def time(self):
        return self.raw["time"]

    @property
    def datetime(self):
        return self.raw["datetime"]

    @property
    def client(self):
        return self.raw.get("user", "")

    @property
    def hostname(self):
        return self.raw.get("hostname", "")

    @property
    def subsystem(self):
        return self.raw.get("subsystem", "")

    @property
    def ssid(self):
        return self.raw.get("ssid", "")
